#include "chil_global.h"

#define MAX_DIFF_V 5
#define MAX_DIFF_PHIDP 40
#define MIN_PHIDP -20

#define CHIL_SQRDIFF 40
#define CLEAR_PIXEL(igate) \
  for (j=0;j<ray->ntotal;j++) *(ray->data[j]+igate) = chil_no_signal_value
#define CHIL_DIFF(answer, ifield, igate) \
   tmpfloat = \
   (*(ray->data[ifield]+igate+1))-(*(ray->data[ifield]+igate)); \
   answer=tmpfloat*tmpfloat

int
CHILRemoveNoiseDoppler(Ray *ray, float threshold)
{
  int iv1, iv2, iphi_dp;

  iv1 = CHILFindField(ray, V1);
  iv2 = CHILFindField(ray, V2);
  iphi_dp = CHILFindField(ray, PHIDP);

  if (iv1 < 0 || iv2 < 0 || iphi_dp < 0) {
    return CHIL_NOTDONE;
  }
  else {
    int i, j;
    float diff1_v2, diff1_phi_dp, tmpfloat;
    CHIL_DIFF(diff1_v2, iv2, 0);
    CHIL_DIFF(diff1_phi_dp, iphi_dp, 0);
    for (i=1 ; i < ray->ngates-1 ; ++i) {
      float diff2_v2, diff2_phi_dp;
      CHIL_DIFF(diff2_v2, iv2, i);
      CHIL_DIFF(diff2_phi_dp, iphi_dp, i);
      if (diff1_v2*CHIL_SQRDIFF + diff1_phi_dp > threshold
	  && diff2_v2*CHIL_SQRDIFF + diff2_phi_dp > threshold) {
	CLEAR_PIXEL(i);
	if (i == 1) {
	  CLEAR_PIXEL(0);
	}
	else if (i == ray->ngates-2) {
	  CLEAR_PIXEL(ray->ngates-1);
	}
      }
      diff1_v2 = diff2_v2;
      diff1_phi_dp = diff2_phi_dp;
    }
  }
  return CHIL_OK;
}


/*  */
int
CHILDopplerClean(Ray *ray)
{
  float *phi_dp, *v;
  float vfold;
  int i, j;
  int iPhiDP = CHILFindField(ray, PHIDP);
  int iV;
  char bad_pixel = 1;

  if (iPhiDP < 0) {
    /* No Phi DP found, so can't clean data */
    return CHIL_NOTFOUND;
  }

  /* Prefer to use V2 (because of the larger folding velocity) */
  /* Can't use both because their characteristics in noise are not independent */
  iV = CHILFindField(ray, V2);
  if (iV >= 0 && ray->use_v2fold) {
    vfold = ray->v2fold;
  }
  else {
    iV = CHILFindField(ray, V1);
    if (iV < 0 || !ray->use_v1fold) {
      return CHIL_NOTDONE;
    }
    else {
      vfold = ray->v1fold;
    }
  }

  phi_dp = ray->data[iPhiDP];
  v = ray->data[iV];

  /* Loop through gates */
  for (i = 1; i < ray->ngates; i++) {
    float diff_phi_dp = phi_dp[i] - phi_dp[i-1];
    float diff_v = v[i] - v[i-1];

    /* Remove folding effects */
    if (diff_v > vfold) {
      diff_v -= vfold * 2;
    }
    else if (diff_v < -vfold) {
      diff_v += vfold * 2;
    }

    if (phi_dp[i-1] < MIN_PHIDP) {
      CLEAR_PIXEL(i-1);
    }
    else if (diff_v > MAX_DIFF_V || diff_v < -MAX_DIFF_V
	|| diff_phi_dp > MAX_DIFF_PHIDP || diff_phi_dp < -MAX_DIFF_PHIDP) {
      if (bad_pixel) {
	CLEAR_PIXEL(i-1);
      }
      bad_pixel = 1;
    }
    else {
      bad_pixel = 0;
    }
  }

  if (bad_pixel) {
    CLEAR_PIXEL(ray->ngates-1);
  }

  return CHIL_OK;
}
